package control;

import java.util.ArrayList;

import datamodel.DataSet;

/**
 * Abstract class representing an operation in the temporal text mining
 * process. All operations must extend this class.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public abstract class Operation {

    private boolean logResult;
    
    /**
     * Method for setting wether the result of the operation should be
     * logged.
     * 
     * @param logResult True if logging
     */
    public void setLogResult(boolean logResult) {
        this.logResult = logResult;
    }
    
    /**
     * Method for getting wether the result of the operation should be logged.
     * 
     * @return True if logging
     */
    public boolean getLogResult() {
        return logResult;
    }
    
    /**
     * Method performing the operation. The dataset is given as a parameter
     * and all changes in this dataset should be stored within this structure. 
     * 
     * @param dataSet The dataset
     */
    public abstract void performOperation(DataSet dataSet);
    
    /**
     * Method for getting the properties used in the operation. Method is used 
     * to present the properties in a window to let the user change them as he
     * or she likes. Each property should be a OperationProperty.
     * 
     * @return List of properties
     */
    public abstract ArrayList getProperties();
    
    /**
     * Method for setting the properties used in the operation. Method is used
     * to set the properties of the operation after the user has changed them
     * in a separate window.
     * 
     * @param properties List of properties
     */
    public abstract void setProperties(ArrayList properties);
}
