package dataloader;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Class implementing the graphical user interface of the document loader.
 * This interface specifies a config file and a destination file to which 
 * all documents should be parsed and stored.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class DocLoader extends JFrame {
    
    private JButton config;
    private JButton destination;
    private JButton load;
    private JTextField configFile;
    private JTextField destinationFile;
    
    /**
     * Constructor creating a new interface and starting the load operation.
     */
	public DocLoader() {
	    super("DocLoader");

        addComponents();
        addListeners();
        
        /* Standard frame settings */
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();
        setLocationRelativeTo(null);
        setVisible(true);
	}
    
    /**
     * Method adding all components to the interface.
     */
    private void addComponents() {
        JPanel panel = new JPanel(new GridLayout(3, 3));
        
        panel.add(new JLabel("Config file: "));
        panel.add(configFile = new JTextField());
        panel.add(config = new JButton("Browse"));
        panel.add(new JLabel("Destination file: "));
        panel.add(destinationFile = new JTextField());
        panel.add(destination = new JButton("Browse"));
        panel.add(new JLabel(""));
        panel.add(new JLabel(""));
        panel.add(load = new JButton("Load documents"));   

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(Box.createVerticalStrut(10), BorderLayout.NORTH);
        getContentPane().add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
        getContentPane().add(panel, BorderLayout.CENTER);
        getContentPane().add(Box.createHorizontalStrut(10), BorderLayout.EAST);
        getContentPane().add(Box.createHorizontalStrut(10), BorderLayout.WEST);
    }
    
    /**
     * Method adding all listeners 
     */
    private void addListeners() {
        destination.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                int returnVal = chooser.showSaveDialog(null);
                String path;
                
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    path = chooser.getSelectedFile().getPath();
                    destinationFile.setText(path);
                }
            }}); 
        config.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                int returnVal = chooser.showOpenDialog(null);
                String path;
                
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    path = chooser.getSelectedFile().getPath();
                    configFile.setText(path);
                }
            }}); 
        load.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(null,
                        "The loading will now execute. Please be patient, this " +
                        "may take a few minutes.",
                        "DocLoader",
                        JOptionPane.PLAIN_MESSAGE);
                
               try {
                new XMLParseren(configFile.getText(), destinationFile.getText());
               } catch (Exception ee) {
                   System.out.println("Error during loading of documents.");
               }

               JOptionPane.showMessageDialog(null,
                       "Loading documents finished!",
                       "DocLoader",
                       JOptionPane.PLAIN_MESSAGE);
               //End program
               System.exit(0);
            }}); 
    }
}
