package dataloader;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import com.sun.org.apache.xerces.internal.parsers.SAXParser;

import datamodel.DataSet;

/**
 * A class that reads and loads the data set containing all the news.
 * 
 * @author Trond ivind Eriksen and Kjell-Inge Skogstad
 */
public class FileHandler {
	
	/**
	 * The method parses the XML file tokenized.xml, and loads the data set. 
	 * @param location
	 * @return ds The data set
	 * @throws Exception
	 */
	public static DataSet loadDataSet(String location) throws Exception {
		
		/* Building the dataset */
		//File readFile = new File(location);
		//XMLParseren xmlpen = new XMLParseren(readFile, true);
		
		/* Parsing the file: tokenized.xml */
		SAXBuilder parser = new SAXBuilder();
		//Document doc = parser.build("tokenized.xml");
		Document doc = parser.build(location);
		
		/* Getting the data from tokenized.xml */
		/* STRUCTURE OF THE XML FILE: tokenized.xml
		 <tokenized>
		 <dataSet name="FTnews" time="dataSetTiden" language="English">
		 <text tid="tidID">
		 irish racing magnates buy 10% stake in united....
		 </text>
		 */
		Element root = doc.getRootElement();	//<tokenized/>
		Element datasettet = root.getChild("dataSet");
		Element text = datasettet.getChild("text");
		List allTexts = datasettet.getChildren("text");
		Attribute name = datasettet.getAttribute("name");
		String name_string = name.getValue();
		Attribute language = datasettet.getAttribute("language");
		int languageInt;
		
		if ("English".equals(language.getValue())) {
			languageInt = DataSet.LANGUAGE_ENGLISH;
		} else {
			languageInt = DataSet.LANGUAGE_NORWEGIAN;
		}
		
		/* Creating the data set */
		String timeGranulity = "days";
		DataSet ds = new DataSet(name_string, timeGranulity, languageInt);
		for (int i = 0; i < allTexts.size(); i++) {
			String tiden = ((Element) allTexts.get(i)).getAttribute("tid").getValue();
			int int_tiden = Integer.valueOf(tiden).intValue();
			ds.addText(int_tiden, ((Element) allTexts.get(i)).getText());
		}
        return ds;
    }
	
	/**
	 * A method that stores the data set.
	 * @param location The location
	 * @param dataSet The data set
	 */
	public static void storeDataSet(String location, DataSet dataSet) {
		//not implemented yet
	}
	
	/**
	 * A method that stores the report.
	 * @param location The location
	 * @param report The report
	 */
	public static void storeReport(String location, String report) {
		try {
			FileWriter writer = new FileWriter(location);
			
			writer.write(report);
			writer.close();
		} catch (IOException e)  {
			e.printStackTrace();
		}
	}
}
