package dataloader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * The class loads an HTML file or an URL to an HTML site and strips this 
 * for HTML tags.
 * 
 * @author Trond ivind Eriksen and Kjell-Inge Skogstad
 */
public class HTMLStripper {
	
	private File [] files;	
	private String url;
	private String orginalFile = "";
	private String pBoundary = "" ;
	private String sBoundary = "";
	private XMLParseren xmlP ;
	private KMP kmp;
	private ArrayList newsList;
	
	/** 
	 * The constructor that uses an URL and parametres from the configuration file.
	 * @param url String
	 * @param sBoundary String
	 * @param pBoundary String
	 * @param allsmall String
	 * @param xmlP XMLParser
	 */
	public HTMLStripper(String url, String sBoundary, String pBoundary, String allsmall, XMLParseren xmlP){
		this.url = url;
		this.pBoundary = pBoundary;
		this.sBoundary = sBoundary;
		this.xmlP = xmlP;
		newsList = new ArrayList();
		
		try {			
			orginalFile = findSource(url);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		parseFile(orginalFile);
	}
	
	/**
	 * The constructor that loads an HTML site from file.
	 * @param filename
	 * @param xmlP
	 * @param sBoundary
	 * @param pBoundary
	 */
	public HTMLStripper(String filename, XMLParseren xmlP, String sBoundary, String pBoundary){
		
		File file = new File(filename);
		this.url = file.getName();
		this.xmlP = xmlP;
		this.pBoundary = pBoundary;
		this.sBoundary = sBoundary;
		if(file.isDirectory()){//checks if a catalogue
			System.out.println("Filen er en directory");
			files = file.listFiles();//gets the files that are in this catalogue
			
			for(int i = 0; i<files.length; i++){
				new HTMLStripper(files[i].getAbsolutePath(),  xmlP, sBoundary, pBoundary);
			}
		}
		
		else{//if not a catalogue
			System.out.println("Filen er en fil");
			try {
				orginalFile = readFile(file);
			} catch (IOException e) {
				e.printStackTrace();
			}
			parseFile(orginalFile);
		}
	}
	
	/**
	 * Method that gets the original file.
	 * @return The original file.
	 */
	public String getOriginalFile(){
		try {
			return findSource(url);
		} catch (MalformedURLException e) {
			e.printStackTrace();	
			return e.getMessage();
			
		} catch (IOException e) {
			e.printStackTrace();	
			return e.getMessage(); 		
		}
	}
	
	/**
	 * Method that collects metadata from file.
	 * @param file The input file.
	 * @return metadata
	 */
	public ArrayList getMeta(String file){
		ArrayList meta = new ArrayList();
		int start = file.indexOf("meta name=\"keywords\" content=\"");
		int slutt = file.indexOf("\">", start);
		String keyWords = file.substring(start+30,slutt);
		StringTokenizer st = new StringTokenizer(keyWords,",");
		while(st.hasMoreTokens()){
			meta.add(st.nextToken());
		}
		start = file.indexOf("<meta name=\"description\" content=\"");
		slutt = file.indexOf("\">", start);
		keyWords = file.substring(start+31,slutt);
		StringTokenizer st2 = new StringTokenizer(keyWords,",");
		while(st2.hasMoreTokens()){
			meta.add(st.nextToken());
		}
		return meta;
	}
	
	/**
	 * This method calls all the different methods that parse file.
	 * @param file String
	 */
	public void parseFile(String file){
		
		String tagsRemoved = "";
		ArrayList sourceList = searchFrame(file);
		String nyhet = searchNews(file);
		String ferdigFil = file.toLowerCase();
		String title = searchTitle(ferdigFil);

		String strippedFrame ;

		orginalFile = ferdigFil;
		tagsRemoved = strip(orginalFile);
		tagsRemoved = stripWhiteSpace(tagsRemoved);
		
		for(int i = 0; i< sourceList.size(); i++){
			strippedFrame =parseFrame((String)sourceList.get(i));
			tagsRemoved += strippedFrame;
		}
		
		try {
			xmlP.makeXmlTokenizer(title, url,tagsRemoved );
			//xmlP.makeXmlTokenizer2(title, url);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Another method that calls all the different methods that parse the file.
	 * @param file String
	 */
	public String parseFilen(String file){
		String tagsRemoved = "";
		ArrayList sourceList = searchFrame(file);
		String ferdigFil = file.toLowerCase();

		String title = searchTitle(ferdigFil);
		String strippedFrame ;
		
		orginalFile = ferdigFil;
		tagsRemoved = strip(orginalFile);
		tagsRemoved = stripWhiteSpace(tagsRemoved);
		
		for(int i = 0; i< sourceList.size(); i++){
			strippedFrame =parseFrame((String)sourceList.get(i));
			tagsRemoved += strippedFrame;
		}
		try {
			//xmlP.makeXmlTokenizer(title, url,tagsRemoved );
			String text = title+tagsRemoved;
			xmlP.makeXmlTokenizer2(text, url);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		}
		return orginalFile;
	}
	
	
	/**
	 * The method calls the methods that have to be called to parse the content
	 * in a frame.
	 * @param frame String
	 * @return String
	 */
	private String parseFrame(String frame) {
		String tagsRemovedFromeFrame  = "";
		ArrayList sourceList = searchFrame(frame);
		//frame = frame.toLowerCase();
		frame = letterStripping(frame);
		frame = stripSpecialChar(frame);
		tagsRemovedFromeFrame = strip(frame);
		tagsRemovedFromeFrame = stripWhiteSpace(tagsRemovedFromeFrame);
		
		return tagsRemovedFromeFrame;
	}
	
	/**
	 * The method reads the file and puts it in a string variable.
	 * @param filename File
	 * @return String
	 * @throws java.io.IOException
	 */
	public String readFile(File filename) throws IOException{
		BufferedReader in = new BufferedReader(new FileReader(filename));
		String file = "";
		String temp;
		
		while((temp = in.readLine())!=null){	
			file+=temp;	
			file+=" ";	
		}
		return file;
	}
	
	/**
	 * The method finds the title to the HTML site.
	 * @param file String
	 * @return String
	 */
	public String searchTitle(String file){	
		String title = "";
		
		int start = file.indexOf("<title>");
		int slutt = file.lastIndexOf("</title>");
		
		if(start==-1||slutt==-1){		
			return title;		
		}
		else{	
			title = file.substring(start+7,slutt);
			title = stripSpecialChar(title);
			title = letterStripping(title);
			return title;
		}
	}
	
	/**
	 * The method finds a news in the HTML site.
	 * @param file String
	 * @return String
	 */
	public String searchNews(String file){
		
		String news = "";
		String news_title = "";
		String news_text = "";
		
		/* Retrieving the main news, and puts it in newsList */
		int news_bigheadline  = file.toLowerCase().indexOf("bigheadline");
		int slutt_bigheadline = file.indexOf("</a>",news_bigheadline);
		int start_hovednyhet  = file.indexOf("<br>",slutt_bigheadline);
		int slutt_hovednyhet  = file.indexOf("<span",start_hovednyhet);
		news_title = file.substring(news_bigheadline+13,slutt_bigheadline);	//news title
		news_text = file.substring(start_hovednyhet+4,slutt_hovednyhet);
		news = news_title+". "+news_text;	//konkatenerer
		
		/* Stripping the news  */
		news = stripSpecialChar(news);
		news = letterStripping(news);
		//news = strip(news);
		//news = stripWhiteSpace(news);
	
		/* Adding in the Arraylist: newsList */
		newsList.add(0, news);
		
		/* Getting the rest of the news and putting them in newsList */
		String pattern = "smallHeadline";
		kmp = new KMP();					//String-matching algorithm
		int[] pro = kmp.kmp(file, pattern); //(String, String)
		
		for(int i=0;i<pro.length-1;i++){

			/*
			class="smallHeadline">Croatia wins incentive for EU entry 
			talks</a><br>Balkan state must improve its record on war crimes 
			*/
			int slutt_smallheadline = file.indexOf("</a><br>",pro[i]);
			
			int start_nyhetstekst = file.indexOf("<br>",slutt_smallheadline);
			int slutt_nyhetstekst = file.indexOf("<span",start_nyhetstekst);
			
			news_title = file.substring(pro[i]+15,slutt_smallheadline);	//the news t
			news_text = file.substring(start_nyhetstekst+4,slutt_nyhetstekst);
			news = news_title+". "+news_text;	//concatenates
			
			/* Stripping the news */
			news = stripSpecialChar(news);
			news = letterStripping(news);
			news = strip(news);
			//news = stripWhiteSpace(news);
			
			/* Removing duplicates of news ("inside" other news)*/
			String[] splittaRead = news.split("Read  ");
			String[] deligen = splittaRead[0].split("2003  ");
			newsList.add(i+1, deligen[0]);	//index 0 used by main news
		}
		return news;
	}
	
	/**
	 * Method that gets the Arraylist of news
	 * @return newsList
	 */
	public ArrayList getNewsList(){
		return newsList;
	}
	
	/**
	 * Method that finds frames in the HTML file.
	 * @param file String
	 * @return ArrayList
	 */
	public ArrayList searchFrame(String file){	
		if(file == null){	
			System.out.println("file is null");	
		}
		String frame = "";
		ArrayList frameList = new ArrayList();
		
		file = file.replaceAll("\\<(FRAME|frame) (SRC|src) =\"(.*?)\".+?\\>", ""+"$1"+ "L" );
		
		while((file.indexOf(""))!= -1 || (file.indexOf("L"))!= -1){
			int first = file.indexOf("");
			int last = file.indexOf("L");
			frame = file.substring(first+1, last);
			
			file = file.replaceFirst("" + frame + "L", " ");
			
			if(frame.startsWith("http")){	
				frameList.add(frame);	
			}
			else{
				frame = url + frame;
				frameList.add(frame);
			}
			
		}
		String source;
		ArrayList sourceList = new ArrayList();
		for(int i = 0; i < frameList.size(); i++){
			try {
				source= findSource((String)frameList.get(i));
				sourceList.add(source);
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return sourceList;
	}
	
	
	/**
	 * The method finds the source code to an URL.
	 * @param urlForFrame String
	 * @return String
	 * @throws java.net.MalformedURLException
	 * @throws java.io.IOException
	 */
	public String findSource(String urlForFrame) throws MalformedURLException,  IOException{
		
		URL frameUrl = new URL(urlForFrame);
		Object source = frameUrl.getContent();
		
		InputStream is = (InputStream)source;
		
		BufferedReader br = new BufferedReader(new InputStreamReader(frameUrl.openStream()));
		String content = "";
		String linje = br.readLine();
		while(linje != null){
			content = content.concat(linje);
			linje = br.readLine();
		}
		return content;
	}
	
	/**
	 * The method removes most HTML tags and spaces.
	 * @param file String
	 * @return file without HTML tags
	 */
	public String strip(String file){
		
		file = file.replaceAll("(\\p{Space}){2,}", " ");
		//file = file.replaceAll("\\s{2,}", " ");
		file = file.replaceAll("<a href=\"http://(.*?)\".+?/a>", " "+"http://$1"+ " " );
		file = file.replaceAll("<a href=\"#(.*?)\".+?/a>", " "+url+"#$1"+ " " );
		file = file.replaceAll("<a href=\"(.*?)\".+?/a>", " "+url+"$1"+ " " );
		file = file.replaceAll("(<script.+?</script>)+", "");
		
		file = file.replaceAll("<p>", pBoundary);
		file = file.replaceAll("<[^>]*>", " ");
		file = file.replaceAll("</[^>]*>", " ");
		file = file.replaceAll("(<meta.*>)+?","");
		
		file = file.replaceAll("#.*\\{[^\\}]*\\}"," "); // tar stylesheet oppmerking
		file = letterStripping(file);
		
		file = stripSpecialChar(file);
		
		file = file.replaceAll("mailto:(.*?)", "$1");
		
		char[] tegn = {',', ';', '!'};
		
		for(int i = 0; i < tegn.length ; i++){
			file = file.replaceAll((String.valueOf(tegn[i])), sBoundary);
		}
		
		file = file.replaceAll("\"", " ");
		file = file.replaceAll("\'", " ");
		return file;
	}
	
	
	/**
	 * The method removes special characters that might occur in HTML.
	 * @param file String
	 * @return file
	 */
	public String stripSpecialChar(String file){
		file = file.replaceAll("&nbsp;", " ");
		file = file.replaceAll("&lt;", " ");
		file = file.replaceAll("&gt;", " ");
		file = file.replaceAll("&copy;", " ");
		file = file.replaceAll("&#[^;];"," ");
		file = file.replaceAll("&#150", " ");//this is a hyphen: -
		file = file.replaceAll("&", " ");
		file = file.replaceAll("#" , " ");
		file = file.replaceAll("\\|", " ");
		file = file.replaceAll("-", " ");
		file = file.replaceAll("\"", " ");
		file = file.replaceAll("\\(", " ");
		file = file.replaceAll("\\)", " ");
		return file;
	}
	
	/**
	 * The method removes all whiteSpaces that are superfluous.
	 * @param file String
	 * @return String
	 */
	public String stripWhiteSpace(String file){
		char[] liste = file.toCharArray();
		
		for (int i = 0; i < (liste.length -1); i++){
			if(Character.isWhitespace(liste[i])|| liste[i] == 'S' ){
				if(Character.isWhitespace(liste[i+1])|| liste[i+1] == 'S'){	
					liste[i] = '';
				}
			}	
		}
		
		String newfile = String.valueOf(liste );
		newfile = newfile.replaceAll("", "");
		newfile = newfile.replaceAll("S", " ");
		return newfile;
	}

	/**
	 * The method changes letters that have special characters in HTML, to 
	 * regular letters.
	 * @param file String
	 * @return String
	 */
	public String letterStripping(String file){	
		//fransk
		file = file.replaceAll("&agrave;", "");
		file = file.replaceAll("&acirc;", "");
		file = file.replaceAll("&ccedil;", "");
		file = file.replaceAll("&egrave;", "");
		file = file.replaceAll("&eacute;", "");
		file = file.replaceAll("&ecirc;", "");
		file = file.replaceAll("&euml;", "");
		file = file.replaceAll("&icirc;", "");
		file = file.replaceAll("&iuml;", "");
		file = file.replaceAll("&ocirc;", "");
		file = file.replaceAll("&aelig;", "");
		file = file.replaceAll("", "");
		file = file.replaceAll("&ugrave;", "");
		
		//tysk
		file = file.replaceAll("&auml;", "");
		file = file.replaceAll("&ouml;", "");
		file = file.replaceAll("&uuml;", "");
		file = file.replaceAll("&szlig;", "");
		
		return file;
	}
	
	
	/**
	 * The main method that starts everything.
	 * @param args String
	 */
	public static void main(String [] args){
		//XMLParseren xmlP = new XMLParseren(new File("s:\\Dokvalt\\config.xml"), false);
		XMLParseren xmlP = new XMLParseren(new File("s:\\Dokvalt\\config.xml"), false);
	}
}
