package datamodel;

import java.util.ArrayList;

/**
 * This class implements a data set. A data set contains any number of 
 * documents. All defined operations perform changes on this data structure.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class DataSet {

    /* Languages supported */
    public static final int LANGUAGE_ENGLISH = 0;
    public static final int LANGUAGE_NORWEGIAN = 1;
    
    private String name;
    private ArrayList texts;
    private int language;

    public boolean isPartOfSpeechPerformed() {
        return partOfSpeech;
    }

    public void setPartOfSpeechPerformed(boolean partOfSpeech) {
        this.partOfSpeech = partOfSpeech;
    }

    public boolean isCollocationPerformed() {
        return collocation;
    }

    public void setCollocationPerformed(boolean collocation) {
        this.collocation = collocation;
    }

    public boolean isWsdPerformed() {
        return wsd;
    }

    public void setWsdPerformed(boolean wsd) {
        this.wsd = wsd;
    }

    private String timeGranulity;

    /* Boolean variable to make sure the proper operations are performed */
    private boolean partOfSpeech=false;
    private boolean collocation=false;
    private boolean wsd=false;
    
    /**
     * Constructor for the class. Creates a new data set.
     * 
     * @param name The name
     * @param timeGranulity The time granulity 
     */
    public DataSet(String name, String timeGranulity, int language) {
        this.name = name;
        this.timeGranulity = timeGranulity;
        this.language = language;
        texts = new ArrayList();
    }
    
    /**
     * A method that adds a text cocument to the data set.
     * 
     * @param tid The timestamp of the document
     * @param text The text
     */
    public void addText(int tid, String text) {
        texts.add(new Text(tid, text));
    }
    
    /**
     * A method that gets the name.
     * 
     * @return name The name
     */
    public String getName() {
        return name;
    }
    
    /**
     * Method that returns the time granulity used in the data set.
     * 
     * @return The time granulity
     */
    public String getTimeGranulity() {
        return timeGranulity;
    }
    
    /**
     * A method that gets the language of a document.
     * 
     * @return language The language
     */
    public int getLanguage() {
        return language;
    }
    
    /**
     * Method for getting a string representation of the dataset language
     * 
     * @return The dataset language
     */
    public String getLanguageStr() {
        if (language == LANGUAGE_ENGLISH) {
            return "English";
        } else if (language == LANGUAGE_NORWEGIAN) {
            return "Norwegian";
        } else {
            return "Unknown";
        }
    }
    
    /**
     * A method that returns all text documents in the data set.
     * 
     * @return texts The texts
     */
    public ArrayList getTexts() {
        return texts;
    }
    
    /**
     * Tests if terms have been extracted.
     * 
     * @return True if terms are extracted
     */
    public boolean termExtracted() {
        for (int i = 0; i < texts.size(); i++) {
            if (((Text)texts.get(i)).hasTerms()) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * A method that returns the number of texts in the data set.
     * @return nofTexts
     */
    public int getNofTexts() {
    	return texts.size();
    }
 
    /**
     * Method returning a string representation of the dataset.
     * 
     * @return string The string
     */
    public String toString() {
        String string = "";
        
        /* Tests on size of the data set */
        if (texts.size() > 0) {
            string = ((Text)texts.get(0)).toString();
            for (int i = 1; i < texts.size(); i++) {
                string += "\n" + ((Text)texts.get(i)).toString();
            }
        }
        return string;
    }
}
