package datamodel;

/**
 * Class representing a term or token. All texts contains a set of terms or 
 * tokens. This class also hold information on frequency and weight of the 
 * term.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Term implements Comparable {

    private String term;
    private int frequency;
    private double weight;
    
    /**
     * Constructor that creates a new term.
     * 
     * @param term The term
     * @param frequency The frequency
     */
    public Term(String term, int frequency, double weight) {
        this.term = term;
        this.frequency = frequency;
        this.weight = weight;
    }
    
    /**
     * Method that returns the term.
     * 
     * @return The term
     */
    public String toString() {
        return term;
    }
    
    /**
     * Method that sets the frequency.
     * 
     * @param frequency The frequency
     */
    public void setFrequency(int frequency) {
        this.frequency = frequency;
    }
    
    /**
     * Method that returns the frequency of the term.
     * 
     * @return The frequency
     */
    public int getFrequency() {
        return frequency;
    }
    
    /**
     * Method that returns the weight of the term.
     * 
     * @return The weight
     */
    public double getWeight() {
        return weight;
    }
    
    /**
     * Method for setting the weight of the term.
     * 
     * @param weight The new weight
     */
    public void setWeight(double weight) {
        this.weight = weight;
    }

    /**
     * Method for comparing the weights of two terms.
     * 
     * @param arg0 Term to compare with
     * @return Decision
     */
    public int compareTo(Object arg0) {
        Term t2 = (Term) arg0;
        
        if (weight < t2.getWeight()) {
            return 1;
        } else if (weight > t2.getWeight()) {
            return -1;
        }
        return 0;
    }
}
