package datapreparation.language;

import java.io.*;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * A class that decides the Language of a document (news)
 * @author trondoiv
 *
 */
public class LanguageDecider{
	
	/** A hashmap for the most common words in English  */
	HashMap commonwords_eng;
	/** A hashmap for the most common words in Norwegian */
	HashMap commonwords_nor;
	
	/** A static integer for the English most common words */
	public static final int ENGLISH = 0;
	/** A static integer for the French most common words */
	public static final int NORWEGIAN = 1;
	/** A static integer if the language is undecided */
	public static final int UNDECIDED = 2;
	
	/**
	 * A constructor for the class. Loads the most common words files of the three languages.
	 *
	 */
	public LanguageDecider(){
		loadEng(getEngDefaultName());
		loadNor(getNorDefaultName());
	}

	/**
	 * Retrieves the filname of the most common words in English.
	 * 
	 * @return the English default filename for most common words.
	 */
	public String getEngDefaultName() {
		return "mfwords_en.txt";
	}
	
	/**
	 * Retrieves the filname of the most common words in Norwegian.
	 * 
	 * @return the Norwegian default filename for most common words.
	 */
	public String getNorDefaultName() {
		return "mfwords_no.txt";
	}
	
	/**
	 * A method that loads a file of the most common English words, and puts them in the HashMap
	 * @param filename the filename of the most common words in English
	 * @return Return true if load is ok, false otherwise
	 */
	public boolean loadEng(String filename)
	{
		BufferedReader br = null;
		try{
			/** From URL */
			//URL url_eng = new URL("http://stud.ntnu.no/groups/prosjekt_temporalmining/mfwords_en.txt");			
			//br = new BufferedReader(new InputStreamReader(url_eng.openStream()));
			br = new BufferedReader(new FileReader(filename));
			commonwords_eng = new HashMap(1000);
			String splitString;
			String[] readword;
	
			splitString = br.readLine();
			int counter = 1;
			while(counter < 9){
				readword = splitString.split("\\s");
				//readword[0] er: rank:
				//readword[1] er: 1..8
				//readword[2] er: ordet
				splitString = br.readLine();
				commonwords_eng.put(readword[2],readword[2]);
				counter++;
			}
		}
		catch(Exception e){
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * A method that loads a file of the most common French words, and puts them in the HashMap
	 * @param filename the filename of the most common words in French
	 * @return Return true if load is ok, false otherwise
	 */
	public boolean loadNor(String filename)
	{
		BufferedReader br = null;
		try{
			/** From URL - commente */
			//URL url_eng = new URL("http://stud.ntnu.no/groups/prosjekt_temporalmining/mfwords_no.txt");			
			//br = new BufferedReader(new InputStreamReader(url_eng.openStream()));
			br = new BufferedReader(new FileReader(filename));
			commonwords_nor = new HashMap(1000);
			String splitString;
			String[] readword;
	
			splitString = br.readLine();
			int counter = 1;
			while(counter < 9){
				readword = splitString.split("\\s");
				splitString = br.readLine();
				commonwords_nor.put(readword[2], readword[2]);
				counter++;
			}
		}
		catch(Exception e){
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	
	/**
	 * A method that checks which language a given sentence is. The sentence is the title in a txt-document.
	 * @param sentence the sentence
	 * @return the language of the document
	 */
	public int checkSentenceInLanguage(String sentence){
		
		int english_counter = 0;
		int norwegian_counter = 0;
		BufferedReader br = null;
		
		try{
			String splitString;
			String[] readword;
			StringTokenizer tk = new StringTokenizer(sentence);
				
				while(tk.hasMoreElements()){
					splitString = tk.nextToken();
					
					if(splitString.equals((String)commonwords_eng.get(splitString))){
						english_counter++;
					}
					
					if(splitString.equals((String)commonwords_nor.get(splitString))){
						norwegian_counter++;
					}
				}
			
			if(english_counter>norwegian_counter){
				//System.out.println("The language is English");
				return ENGLISH;
			}else if(norwegian_counter>english_counter){
				//System.out.println("The language is Norwegian");
				return NORWEGIAN;
			}else{
				System.err.println("The language could not be decided");
				return UNDECIDED;
			}
			
		}//try
		catch(Exception e){
			e.printStackTrace();
			return UNDECIDED;
		}		
	}
	
}//class