package datapreparation.noun;

import java.util.ArrayList;

import datamodel.DataSet;
import datamodel.Text;
import datapreparation.TextOperation;
import control.Control;

/**
 * Class implementing an operation to extrac nouns from the texts in the 
 * dataset. A noun is here given by a word where the first letter is upper 
 * case.
 * 
 * The operations also deals with all words independently. That is, no Noun 
 * groups are formed. If this is needed, code for this is exist but is commented 
 * away.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Noun extends TextOperation {

    private String stringText;
    private ArrayList<String> dividers;

    /**
     * Constructor for the noun operation. Initializes a vector of characters
     * that divide the text into sentences or part of sentences. For example, 
     * , ) ( and so on.
     */
    public Noun() {
        dividers = new ArrayList<String>();
        dividers.add("'");
        dividers.add("");
        dividers.add("(");
        dividers.add("?");
        dividers.add(".");
        dividers.add(",");
        dividers.add("!");
        dividers.add(")");
        dividers.add("");
        dividers.add(":");
        dividers.add(";");
    }

    /**
     * Method for performing the operation.
     */
    public void performOperation(DataSet dataSet) {
        ArrayList<Text> texts = dataSet.getTexts();

        // Extracting nouns from all texts
        for (int i = 0; i < texts.size(); i++) {
            texts.add(i, extractNounsInText((Text)texts.remove(i)));
        }

    }

    /**
     * Metehod for extracting nouns in a specific text.
     * 
     * @param text The text
     * @return The new text
     */
    private Text extractNounsInText(Text text) {
        Text newText = new Text(text.getTid(), text.toString());
        String sentence;

        stringText = text.toString();

        // Finding first sentence
        sentence = findSentence();
        while (!sentence.trim().equals("")) {
            //Extract nouns from sentence
            extractNounsInSentence(sentence, newText);

            //Fetch new sentence
            sentence = findSentence();
        }
        return newText;
    }

    /**
     * Method for identifying and returning the first sentence. This sentence 
     * is then removed from the text string representation.
     * 
     * @return The sentence
     */
    private String findSentence() {
        int index = 200;
        String sentence;
        int temp;

        //Find first divider
        for (int i = 0; i < dividers.size(); i++) {
            temp = stringText.indexOf(dividers.get(i));

            if (temp > 0 && temp < index) {
                index = temp;
            }
        }

        //Extract sentence
        if (index < 200 && index > 0) {
            sentence = stringText.substring(0, index);
            stringText = stringText.substring(index + 1, stringText.length());
            return sentence.trim();
        }

        sentence = stringText;
        stringText = "";
        return sentence.trim();
    }

    /**
     * Method for extracting nouns in a sentence.
     * 
     * @param sentence The sentence
     * @param text The new text
     */
    private void extractNounsInSentence(String sentence, Text text) {
        String words[] = sentence.split(" ");
        String word;
        //String word2;
        //String word3;

        for (int i = 0; i < words.length; i++) {
            word = words[i];

            if (word.length() > 0 && Character.isUpperCase(word.charAt(0))) {
                text.addTerm(word, 1, 0.0);

                /* Code used to identify noun groups of up to three terms
                    if (i + 1 < words.length) {
                    word2 = words[i + 1];
                    
                    if (word2.length() > 0 && Character.isUpperCase(word2.charAt(0))) {
                        text.addTerm(word + " " + word2, 1, 0.0);
                        
                        if (i + 2 < words.length) {
                            word3 = words[i + 2];
                            
                            if (word3.length() > 0 && Character.isUpperCase(word3.charAt(0))) {
                                text.addTerm(word + " " + word2 + " " + word3, 1, 0.0);
                            }
                        }
                    }
                }*/
            }
        }
    }

    /**
     * Method for getting the properties.
     */
    public ArrayList getProperties() {
        return null;
    }

    /**
     * Method for setting the properties.
     */
    public void setProperties(ArrayList properties) {
    }
}
