package datapreparation.stemming;


/**
 * A Norwegian stemmer class (based on Snowball stemming algorithm)
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class NorwegianStemmer implements Stemmer {
	
	/**
	 * An empty constructor to the class
	 */
	public NorwegianStemmer(){}
	
	
	/** First list of suffixes in Norwegian that should be stemmed **/
    String[] stem1 = new String[] {
            "a",      // -1, 1
            "e",      // -1, 1
            "ede",    //  1, 1
            "ande",   //  1, 1
            "ende",   //  1, 1
            "ane",    //  1, 1
            "ene",    //  1, 1
            "hetene", //  6, 1
            "erte",   //  1, 3
            "en",     // -1, 1
            "heten",  //  9, 1
            "ar",     // -1, 1
            "er",     // -1, 1
            "heter",  // 12, 1
            "s",      // -1, 2
            "as",     // 14, 1
            "es",     // 14, 1
            "edes",   // 16, 1
            "endes",  // 16, 1
            "enes",   // 16, 1
            "hetenes",// 19, 1
            "ens",    // 14, 1
            "hetens", // 21, 1
            "ers",    // 14, 1
            "ets",    // 14, 1
            "et",     // -1, 1
            "het",    // 25, 1
            "ert",    // -1, 3
            "ast",    // -1, 1
        };

    	/** Second list of suffixes in Norwegian that should be stemmed **/
        String[] stem2 = new String[] {
            "dt",     // -1, -1 
            "vt",	  // -1, -1
        };

        /** Third list of suffixes in Norwegian that should be stemmed **/
        String[] stem3 = new String[] {
            "leg",    // -1, 1  
            "eleg",   //  0, 1  
            "ig",     // -1, 1  
            "eig",    //  2, 1  
            "lig",    //  2, 1  
            "elig",   //  4, 1  
            "els",    // -1, 1  
            "lov",    // -1, 1  
            "elov",   //  7, 1  
            "slov",   //  7, 1  
            "hetslov",//  9, 1
        };
        
	/**
	 * A method that performs stemming on one word
	 * @param s The word to be stemmed
	 * @return s the stemmed word
	 */
	public String stem(String s) {
		String orig = s;
		
        s = s.trim();
        // Used to protect stemmed forms in search queries.
        // If words marked with * are found, then remove the *, but do
        // not remove more.
        if(s.endsWith("*")) {
            s = s.substring(0,s.length()-1);
        	return s;
        }
        for(int i=0; i<stem1.length; i++)
        {        	
            if(s.endsWith(stem1[i]))
            {
            	s = s.substring(0,s.length()-stem1[i].length());
            	System.out.println(s);
                break;
            }
            
        }
        for(int i=0; i<stem2.length; i++)
        {
            if(s.endsWith(stem2[i]))
            {
                s = s.substring(0,s.length()-stem2[i].length());
                break;
            }

        }
        for(int i=0; i<stem3.length; i++)
        {
            if(s.endsWith(stem3[i]))
            {
                s = s.substring(0,s.length()-stem3[i].length());
                break;
            }

        }

        for(int i=0; i<s.length(); i++)
        {
            char ch = s.charAt(i);
            if(Character.isDigit(ch) || ch=='+' || ch=='%' || ch=='<' || ch=='>')
            {
                s = s.substring(0,i)+s.substring(i+1);
                i--;
                continue;
            }
        }

        if(s.length() <= 1)
        	return orig;
        
        return s;
	}

}//class
