package datapreparation.stemming;

import gui.MainFrame;

import java.util.ArrayList;

import datamodel.DataSet;
import datamodel.Term;
import datamodel.Text;
import datapreparation.TextOperation;
import control.Control;

/**
 * Class implementing the stemming operation. Given the language used in the 
 * dataset, all terms in the dataset is stemmed.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Stemming extends TextOperation {

    private Stemmer stemmer;

    /**
     * Method for performing the operation.
     */
    public void performOperation(DataSet dataSet) {
        ArrayList texts = dataSet.getTexts();

        //Setting stemmer by language
        if (dataSet.getLanguage() == DataSet.LANGUAGE_ENGLISH) {
            stemmer = new EnglishStemmer();
        } else if (dataSet.getLanguage() == DataSet.LANGUAGE_NORWEGIAN) {
            stemmer = new NorwegianStemmer();
        } else {
            MainFrame.getInstance().printMessage("Language not supported. " +
                    "Stemming operation aborted.");
            return;
        }

        //Test if terms are extracted
        if (!dataSet.termExtracted()) {
            MainFrame.getInstance().printMessage("Terms are not extracted. " +
                    "Stemming operation aborted");
            return;
        }

        for (int i = 0; i < texts.size(); i++) {
            texts.add(i, stemWordsInText((Text)texts.remove(i)));
        }
    }

    /**
     * Method for stemming all terms in a specific text.
     * 
     * @param text The text
     * @return The new text
     */
    public Text stemWordsInText(Text text) {
        ArrayList terms = text.getTerms();
        Text newText = new Text(text.getTid(), text.toString());
        String term;

        /* Build new termlist */
        for (int i = 0; i < terms.size(); i++) {
            term = ((Term)terms.get(i)).toString();
            term = stemmer.stem(term);
            newText.addTerm(term, ((Term)terms.get(i)).getFrequency(), 0.0);
        }
        return newText;
    }

    /**
     * Method for getting the properties.
     */
    public ArrayList getProperties() {
        return null;
    }

    /**
     * Method for setting the properties.
     */
    public void setProperties(ArrayList properties) {
    }
}
