package datapreparation.stopword;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * A class that eliminates English stopwords from a document.
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 *
 */
public class EnglishStopwords implements Stopwords {

	/** HashMap of English stopwords **/
	HashMap stopwords_eng;

	/**
	 * A constructor that loads the English stopwords file, and puts the
	 * words in a HashMap
	 */
	public EnglishStopwords(){
		boolean file = loadEng(getEngDefaultName());
        if(!file){
            loadEngFromURL(getEngDefaultName());
        }
    }

	/**
	 * A method that gets the textfile with English stopwords.
	 * @return The textfile with English stopwords.
	 */
	public String getEngDefaultName(){
		return "stopwords_en.txt";
	}

	/**
	 * A method that loads a file of the most common English words, and puts them in the HashMap
	 * @param filename the filename of the most common words in English
	 * @return Return true if load is ok, false otherwise
	 */
	public boolean loadEng(String filename)
	{
		BufferedReader br = null;
		try{
			//URL url_eng = new URL("http://stud.ntnu.no/groups/prosjekt_temporalmining/stopwords_en.txt");
			//br = new BufferedReader(new InputStreamReader(url_eng.openStream()));
			br = new BufferedReader(new FileReader(filename));
			stopwords_eng = new HashMap(1000);

			String stopword;
			stopword = br.readLine();
			int counter=0;

			while(true){
				stopwords_eng.put(stopword,stopword);
				stopword = br.readLine();
				counter++;
				if(stopword == null || stopword.equals("") ||
                        stopword.equals(" ")){
					break;
				}
			}
		}
		catch(Exception e){
			//e.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * A method that loads English stopwords from an URL.
	 * @param filename
	 * @return boolean
	 */
	public boolean loadEngFromURL(String filename)
	{
		BufferedReader br = null;
		try{
			URL url_eng = new URL("http://folk.ntnu.no/olekrif/stopwords_en.txt");
			br = new BufferedReader(new InputStreamReader(url_eng.openStream()));
			stopwords_eng = new HashMap(1000);

			String stopword;
			stopword = br.readLine();
			int counter=0;

			while(true){
				stopwords_eng.put(stopword,stopword);
				stopword = br.readLine();
				counter++;
				if(stopword == null || stopword.equals("") ||
                        stopword.equals(" ")){
					break;
				}
			}
		}
		catch(Exception e){
			e.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * A method that concatenates two strings.
	 * @param string1 The first string to be concatenated
	 * @param string2 The second string to be concatenated
	 * @return The concatenated string (string1+string2)
	 */
	public String stringCat(String string1, String string2){
		return (string1 + " "+ string2);
	}

	/**
	 * Method that removes English stopwords.
	 * @param sentence The sentence that stopwords should be removed from.
	 * @return sentence The sentence, not containing any stopwords.
	 */
	public String removeStopwords(String sentence){
		sentence = sentence.toLowerCase();
		String newSentence="";
		try{
			String splitString;
			StringTokenizer tk = new StringTokenizer(sentence);
			while(tk.hasMoreElements()){
				splitString = tk.nextToken();
				//System.out.println("splitString er:" +splitString);
				if(splitString.equals((String)stopwords_eng.get(splitString))){
					//System.out.println("Et stoppord ble fjernet:" +splitString);
					newSentence = stringCat(newSentence,"");
					newSentence = newSentence.replaceAll("  "," ");
				}
				else{
					newSentence = stringCat(newSentence,splitString);
					newSentence = newSentence.replaceAll("  "," ");
				}
			}
			sentence = newSentence;
		}
		catch(Exception e){
			e.printStackTrace();
			return "";
		}
		return sentence;
	}

}//class
