package datapreparation.stopword;

import gui.MainFrame;

import java.util.ArrayList;

import datamodel.DataSet;
import datamodel.Term;
import datamodel.Text;
import datapreparation.TextOperation;

/**
 * Class implementing the stopword removal operation. Given the language used 
 * in the dataset, all stopwords are removed. Stopwords are given in the files: 
 * stopwords_en.txt and stopwords_no.txt.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Stopword extends TextOperation {

    private Stopwords stopwords;
    private MainFrame mainFrame;
    
    /**
     * Method for performing the operation.
     */
    public void performOperation(DataSet dataSet) {
        ArrayList texts = dataSet.getTexts();
        mainFrame = MainFrame.getInstance();
        
        //Setting stopwords by language
        if (dataSet.getLanguage() == DataSet.LANGUAGE_ENGLISH) {
            stopwords = new EnglishStopwords();
        } else if (dataSet.getLanguage() == DataSet.LANGUAGE_NORWEGIAN) {
            stopwords = new NorwegianStopwords();
        } else {
            MainFrame.getInstance().printMessage("Language not supported. " +
                    "Stopword operation aborted.");
            return;
        }
        
        //Test if terms are extracted
        if (!dataSet.termExtracted()) {
            MainFrame.getInstance().printMessage("Terms are not extracted. " +
                    "Stopword operation aborted");
            return;
        }
        int numberOfTexts = texts.size();
        for (int i = 0; i < texts.size(); i++) {
            texts.add(i, removeStopwordsInText((Text)texts.remove(i)));
            int progress = (int)((double)(i + 1)/(double)numberOfTexts * 100);
            mainFrame.updateOperationProgress(progress);
        }
    }

    /**
     * Method for removing the stopwords in a given text.
     * 
     * @param text The text
     * @return The new text
     */
    public Text removeStopwordsInText(Text text) {
        ArrayList terms = text.getTerms();
        Text newText = new Text(text.getTid(), text.toString());
        String term;
        String result;
        
        /* Build new termlist */
        for (int i = 0; i < terms.size(); i++) {
            term = ((Term)terms.get(i)).toString();

            int posTag = term.indexOf("/");
            String tag="";
            if(posTag!=-1){
                tag = term.substring(posTag, term.length());
            }

            if(!tag.contains("NNP")){
                term = term.substring(0, term.indexOf("/"));
                result = stopwords.removeStopwords(term);
                result = result.trim();
                if (result != null && !result.equals("")) {
                term=term+tag;
                newText.addTerm(term, ((Term)terms.get(i)).getFrequency(),
                        ((Term)terms.get(i)).getWeight());
                }
            }
            else{
                newText.addTerm(term, ((Term)terms.get(i)).getFrequency(),
                        ((Term)terms.get(i)).getWeight());
            }


        }
        return newText;
    }
    
    /**
     * Method for getting the properties.
     */
    public ArrayList getProperties() {
        return null;
    }

    /**
     * Methods for setting the properties.
     */
    public void setProperties(ArrayList properties) {
    }
}
