package datapreparation.tokinize;

import java.util.ArrayList;

import control.OperationProperty;

import datamodel.DataSet;
import datamodel.Text;
import datapreparation.TextOperation;

/**
 * Class implementing the tokinizing operation. Here all terms are extracted
 * from the text and inserted into the data model.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Tokinize extends TextOperation {
    
    /**
     * Method for performing the operation.
     */
    public void performOperation(DataSet dataSet) {
        ArrayList texts = dataSet.getTexts();
            
        for (int i = 0; i < texts.size(); i++) {
            extractFromText((Text)texts.get(i));
        }
    }

    /**
     * Metehod used to extract terms and insert them into the text model.
     * 
     * @param text
     */
    private void extractFromText(Text text) {
        String[] terms = text.toString().split(" ");        
        text.removeAllTerms();
        for (int i = 0; i < terms.length; i++) {
            terms[i] = terms[i].trim();                     
            terms[i] = clean(terms[i]);    
            
            if (!terms[i].trim().equals("")) {
                text.addTerm(terms[i], 1, 0.0);
            }
        }
    }
    
    /**
     * Method used to clean the terms. Non letters or digits are removed from 
     * start and end of term.
     * 
     * @param term The term
     * @return The cleaned term
     */
    private String clean(String term) {
        //Remove non letters or digits at start of term
        while (term.length() > 0 && !Character.isLetterOrDigit(term.charAt(0))) {
            term = term.substring(1, term.length());
            term = term.trim();
        }
        
        //Remove non letters or digits at end of term
        while (term.length() > 0 && 
                !Character.isLetterOrDigit(term.charAt(term.length() - 1))) {
            term = term.substring(0, term.length() - 1);
            term = term.trim();
        }  
        return term;
    }
    
    /**
     * Method for getting the porperties.
     */
    public ArrayList getProperties() {
        return null;
    }

    /**
     * Method for setting the properties.
     */
    public void setProperties(ArrayList properties) {  
    }
}
