package datapreparation.weighting;

/**
 * Class implementing all weighting functions used in the application.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class WeightHelper {

    /**
     * method implementing the computation of a TfIdf weight.
     * 
     * @param numDocsTotal Total number of documents
     * @param numDocsOcc Number of documents that contain current item
     * @param nOccMax Frequency of the most frequent item in current document
     * @param nOccThisDoc Frequenco of this item in current document
     * @return The weight
     */
    public static double computeTfIdf(int numDocsTotal, int numDocsOcc, 
            int nOccMax, int nOccThisDoc) {        
        double TF = (double)nOccThisDoc / (double)nOccMax;   
        double IDF = Math.log((double)numDocsTotal / (double)numDocsOcc);  
        
        return TF*IDF;    
    }
}