package datapreparation.weighting;

import java.util.ArrayList;

import datamodel.DataSet;
import datamodel.Term;
import datamodel.Text;
import datapreparation.TextOperation;

/**
 * Class implementing the weighting operation. 
 * 
 * Currently only TfIdf weighting present.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Weighting extends TextOperation {

    private TermCollection termCollection;
    private int nofTexts;
    
    /**
     * Method for performing the operation.
     */
    public void performOperation(DataSet dataSet) {
        termCollection = new TermCollection();
        
        termCollection.fillCollection(dataSet);
        
        //Fill local variables
        nofTexts = dataSet.getNofTexts();
        
        //Weight
        for (int i = 0; i < dataSet.getNofTexts(); i++) {
            weightWordsInText((Text)dataSet.getTexts().get(i));
        }  
    }

    /**
     * Method for weighting the terms in a given text.
     * 
     * @param text The text
     */
    private void weightWordsInText(Text text) {
        ArrayList terms = text.getTerms();
        Term term;
        
        for (int i = 0; i < terms.size(); i++) {
            term = (Term)terms.get(i);
            
            term.setWeight(WeightHelper.computeTfIdf(
                    nofTexts, 
                    termCollection.getNofTexts(term.toString()), 
                    1, 
                    term.getFrequency()));
        }
    }
    
    /**
     * Method for getting the properties.
     */
    public ArrayList getProperties() {
        return null;
    }

    /**
     * Method for setting the properties.
     */
    public void setProperties(ArrayList properties) {  
    }
}
