package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import control.Control;

/**
 * Class implementing the application menu.
 * 
 * @author Kjell-Inge Skogstad
 */
public class Menu extends JMenuBar {

    private Control control;
    private JMenuItem exit, help, aboutItem, openDataset, saveDataset, run;
    
    /**
     * Constructor that initializes the menu.
     * 
     * @param control The control instance
     */
    public Menu(Control control) {
        this.control = control;
        
        addComponents();
        addListeners();
        
        //Temporary
        saveDataset.setEnabled(false);
        help.setEnabled(false);
    }
    
    /**
     * Method adding all components to the menu.
     */
    private void addComponents() {
        JMenu file, about, mining;
        
        /* The File menu */
        file = new JMenu("File");       
        
        openDataset = new JMenuItem("Open Dataset");
        openDataset.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, 
                java.awt.event.InputEvent.CTRL_MASK)); 
        file.add(openDataset);
        
        saveDataset = new JMenuItem("Save Dataset");
        saveDataset.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, 
                java.awt.event.InputEvent.CTRL_MASK)); 
        file.add(saveDataset);
        
        file.addSeparator();
        
        exit = new JMenuItem("Exit");
        exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, 
                java.awt.event.InputEvent.CTRL_MASK));
        file.add(exit);
        
        add(file);
        
        /* The mining menu*/
        mining = new JMenu("Temporal Text Mining");        
        
        run = new JMenuItem("Run Process");
        run.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, 
                java.awt.event.InputEvent.CTRL_MASK)); 
        mining.add(run);
        
        add(mining);
        
        /* The About menu */
        about = new JMenu("About");      
        
        help = new JMenuItem("Help");
        help.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, 
                java.awt.event.InputEvent.CTRL_MASK)); 
        about.add(help);
        
        about.addSeparator();
        
        aboutItem = new JMenuItem("About");
        aboutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, 
                java.awt.event.InputEvent.CTRL_MASK));      
        about.add(aboutItem); 
        
        add(about);
    }
    
    /**
     * Method adding all listeners to components in the menu.
     */
    private void addListeners() {
        exit.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                    System.exit(0);
            }});      
        help.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(null, 
                        "Help... ", 
                        "Help", 
                        JOptionPane.INFORMATION_MESSAGE);
            }});     
        aboutItem.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(null, 
                        "TTM Testbench version 0.1 \n\n" +
                            "Developed by Trond ivind Eriksen and " +
                            "Kjell-Inge Skogstad \n" +
                        "Modified by Ole Kristian Fivelstad", 
                        "About", 
                        JOptionPane.INFORMATION_MESSAGE); 
            }});
        openDataset.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser("data");
                int returnVal = chooser.showOpenDialog(null);
                
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    control.openDataSet(chooser.getSelectedFile().getPath());
                }
            }});
        saveDataset.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                int returnVal = chooser.showSaveDialog(null);
                
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    control.saveDataSet(chooser.getSelectedFile().getPath());
                }
            }});
        run.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {
                control.runProcess();
            }});
    }
}
