package gui.panel;

import gui.MainFrame;

import java.awt.BorderLayout;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Class implementing the document selection panel. Selections made here
 * should decide what document report is shown in gui.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class DocPanel extends JPanel {
    
    private MainFrame mainFrame;
    private JSlider slider;
    
    /**
     * Constructor that initializes the panel.
     * 
     * @param mainFrame The frame component
     */
    public DocPanel(MainFrame mainFrame) {
        super(new BorderLayout());
        
        this.mainFrame = mainFrame;
        
        addComponents();
        addListeners();
        
        setBorder(BorderFactory.createTitledBorder("Documents"));
    }

    /**
     * Method adding all components to the panel.
     */
    private void addComponents() {
        slider = new JSlider();
        slider.setMinimum(1);
        slider.setPaintLabels(true);
        slider.setPaintTicks(true);
        
        add(slider);
        
        configSlider(100);
    }
    
    /**
     * Method adding all listeners to the components.
     */
    private void addListeners() {
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent arg0) {
                mainFrame.setDocIndex(slider.getValue() - 1);
            }    
        });
    }
    
    /**
     * Method for setting the position of the slider. Also tells the
     * mainframe to update the gui. 
     * 
     * @param index The new index
     */
    public void setSlider(int index) {
        slider.setValue(index);        
        slider.revalidate();
        mainFrame.setDocIndex(slider.getValue() - 1);
    }
    
    /**
     * Method for configurating the slider by setting maximum.
     * 
     * @param max The maximum
     */
    public void configSlider(int max) {
        slider.setMaximum(max);
        if (max >= 20) {
            slider.setMajorTickSpacing(max / 5);
        } else {
            slider.setMajorTickSpacing(max);
        }
        
        setSlider(1);
    }
}
