package gui.panel;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import control.Control;

/**
 * Class implementing the panel where the user can initialize the 
 * temporal text mining process. Includes a run button and a progress
 * bar.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class RunPanel extends JPanel {

    private Control control;
    private JButton runButton;
    private JProgressBar progress;
    
    /**
     * Constructor creating a new run panel.
     * 
     * @param control The control instance
     */
    public RunPanel(Control control) {
        super(new GridLayout(2,1));
        
        this.control = control;
        
        addComponents();
        addListeners();
        
        setBorder(BorderFactory.createTitledBorder("Run"));
    }
    
    /**
     * Method adding all components.
     */
    private void addComponents() {
        add(runButton = new JButton("Run Process")); 
        
        progress = new JProgressBar(0, 100);
        progress.setStringPainted(true);
        progress.setString("0%");
        progress.setValue(0);
        add(progress);
    }
    
    /**
     * Method used to update the progress bar. The progress is given by a 
     * number between 0 and 100.
     * 
     * @param newProgress The new progress
     */
    public void updateProgress(int newProgress) {
        if (newProgress <= 100 || newProgress >= 0) {
            progress.setString(newProgress + "%");
            progress.setValue(newProgress);
            progress.revalidate();
            progress.repaint();
        }
    }
    
    /**
     * Method adding all listeners to the components.
     */
    private void addListeners() {
        runButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {          
                progress.setString("0%");
                progress.setValue(0);
                
                control.runProcess();
            }
        });
    }
}
