package gui.panel;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import control.Control;
import dataloader.XMLParseren;

/**
 * Class that implements the source selection panel. Here the user can select
 * and view details about the data set.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class SourcePanel extends JPanel {

    private Control control;
    private JButton openButton;
    private JTextField name;
    private JTextField language;
    private JTextField nofTexts;
    private XMLParseren xmlp;
    
    /**
     * Constructor creating a new source panel.
     * 
     * @param control The control instance
     */
    public SourcePanel(Control control) {
        super(new GridLayout(4, 2));
        
        this.control = control;
        
        addComponents();
        addListeners();
        
        setBorder(BorderFactory.createTitledBorder("Dataset"));
    }
    
    /**
     * Method adding all components to the panel.
     */
    public void addComponents() {
        add(new JLabel("Name"));
        add(name = new JTextField());
        add(new JLabel("Language"));
        add(language = new JTextField());
        add(new JLabel("Number of texts"));
        add(nofTexts = new JTextField());
        add(new JLabel(""));
        add(openButton = new JButton("Open"));
        
        /* Setting restrictions on the fields */
        name.setEditable(false);
        language.setEditable(false);
        nofTexts.setEditable(false);
    }
    
    /**
     * Method adding all listeners to the components.
     */
    public void addListeners() {
        openButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) {          
                JFileChooser chooser = new JFileChooser("data");
                int returnVal = chooser.showOpenDialog(null);
                
                if(returnVal == JFileChooser.APPROVE_OPTION) {
                    control.openDataSet(chooser.getSelectedFile().getPath());
                }
            }
        });
    }
    
    /**
     * Method used to set the information about the selected data set.
     * 
     * @param name The name
     * @param language The language
     * @param nofTexts The number of texts
     */
    public void setInfo(String name, String language, String nofTexts) {
        this.name.setText(name);
        this.language.setText(language);
        this.nofTexts.setText(nofTexts);
    }
    
    /**
     * Method used to set the name of the selected data set.
     */
    public void setName(String name){
    	this.name.setText(name);
    }
    
    /**
     * Method used to set the language information about the selected data set.
     * @param language
     */
    public void setLanguage(String language){
    	this.language.setText(language);
    }
    
    /**
     * Method used to set the number of texts of the selected data set.
     * @param nofTexts
     */
    public void setNofTexts(String nofTexts){
    	this.nofTexts.setText(nofTexts);
    }
}
