package modelling.fiti;

import java.text.DecimalFormat;
import java.util.Locale;
import java.math.BigDecimal;

/**
 * Class implementing a association rule. 
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Rule {

    private String rule;
    private double support;
    private double confidence;
    private double similarity;
    private DecimalFormat df;
    private DecimalFormat df2;

    /**
     * Constructor creating a new rule.
     * 
     * @param rule The rule
     * @param support The suppport
     * @param confidence The confidence
     */
    public Rule(String rule, double support, double confidence, double similarity) {
        this.rule = rule;
        this.support = support;
        this.confidence = confidence;
        this.similarity = similarity;
        df = new DecimalFormat("0.00");
        df2 = new DecimalFormat("0.0000");
    }

    /**
     * Method for getting the rule.
     * 
     * @return The rule
     */
    public String getRule() {
        return rule;
    }

    /**
     * Method for getting the support.
     * 
     * @return The support
     */
    public String getSupport() {
        return df.format(support);
    }

    /**
     * Method for getting the confidence.
     * 
     * @return The confidence
     */
    public String getConfidence() {
        return df.format(confidence);
    }

    /**
     * Method for getting the semantic similarity.
     *
     * @return The similarity
     */
    public String getSimilarity(){
       return df2.format(similarity);
    }

    /**
     * Method for getting a string representation of the rule.
     */
    public String toString() {
        return rule + ", confidence: " + getConfidence() + ", support: " + getSupport()
                + ", similarity: " + getSimilarity();
    }
}
