package modelling.fiti.data;

/**
 * Class implementing an item. An item has either a filt index or a term list
 * index. The type provides what kind of index is used.
 * 
 * @author Kjell-Inge Skogstad, Trond ivind Eriksen and Ole Kristian Fivelstad
 */
public class Item {
    
    //Types of indexes used
    public static int TERMLIST_INDEX = 0;
    public static int FILT_INDEX = 1;

    protected int index;
    protected int type;
    
    /**
     * Constructor creating a new item.
     * 
     * @param index Filt of TermList index
     * @param indexType Index type
     */
    public Item(int index, int indexType) {
        this.index = index;
        type = indexType;
    }

    /**
     * Method for getting the index.
     * 
     * @return The index
     */
    public int getIndex() {
        return index;
    }

    /**
     * Method for getting the index type.
     * 
     * @return The index type
     */
    public int getIndexType() {
        return type;
    }
    
    /**
     * Method for getting a clone of this instance.
     */
    public Object clone() {
        return new Item(index,type);
    }
    
    /**
     * Method for testing wether this instance is equal to another.
     */
    public boolean equals(Object o) {
        if (o instanceof Item) {
            Item item = (Item)o;
        
            return (index == item.getIndex()) && (type == item.getIndexType());
        }
        return false;
    }
    
    /**
     * Method returning a printable representation of the item.
     */
    public String toString() {
        return index + ":" + type;
    }
}
