package modelling.fiti.data;

import java.util.ArrayList;

/**
 * List of Items and InterItems. Shall be only one of a kind of each item. 
 * Index of InterItem is given by position in this list. 
 * 
 * ItemList is used so that all itemsets are given a unique index. This is 
 * given by the position in the list. The index is need in communication with 
 * external classes.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class ItemList {
    
    //The instance variable
    public static ItemList INSTANCE;
    
    private ArrayList<Object> itemset;
    private ArrayList clusters;

    public int getSize(){
        return itemset.size();
    }
    /**
     * Constructor creating a new itemlist.
     *
     */
    public ItemList() {
        itemset = new ArrayList<Object>();
        clusters = new ArrayList();
        INSTANCE = this;
    }
    
    /**
     * Method for adding an item. If item is already present only
     * index is returned.
     * 
     * @param item The item
     * @return The index
     */
    public int addItem(Item item) {
        for (int i = 0; i < itemset.size(); i++) {
            if (item instanceof InterItem) {
                if (((InterItem)item).equals(itemset.get(i))) {
                    return i + 1;
                }
            } else {
                if (item.equals((itemset.get(i)))) {
                    return i + 1;
                }
            }
        }        
        itemset.add(item);
        return itemset.size();
    }
    
    /**
     * Method for getting the index of an item.
     * 
     * @param item The item
     * @return The index
     */
    public int indexOfItem(Item item) {
        Item localItem;
        
        for (int i = 0; i < itemset.size(); i++) {
            localItem = (Item)itemset.get(i);
            
            if (item.equals(localItem)) {
              return i + 1;
            }
        }
        return -1;
    }
    
    /**
     * Method for getting an item.
     * 
     * @param i The index
     * @return The item
     */
    public Item getItem(int i) {
        return (Item)itemset.get(i - 1); 
    }
    
    /**
     * Method for getting a printable representation of the itemlist.
     */
    public String toString() {
        return itemset.toString();
    }
}
