package modelling.fiti.data;

import java.util.ArrayList;

/**
 * Class implementing an itemset. An itemset includes a list of items, either 
 * intra or inter, and associated count and support.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class ItemSet {
    
    private ArrayList items;
    private int count;
    private double support;
    
    /**
     * Constructor creating a new itemset. Count and support are set to 0.
     */
    public ItemSet() {
        items = new ArrayList();
        count = 0;
        support = 0;
    }

    /**
     * Method for adding an item. Item is stored as an itemlist index.
     * 
     * @param item The item
     */
    public void addItem(Item item) {
        items.add(new Integer(ItemList.INSTANCE.addItem(item)));
    }
    
    /**
     * Method for getting the itemlist index of item given its index in
     * the itemset.
     * 
     * @param i The itemset index
     * @return The itemlist index
     */
    public int getItemListIndex(int i) {
        return ((Integer)items.get(i)).intValue();
    }
    
    /**
     * Method for removing an item.
     * 
     * @param index The index
     */
    public void removeItem(int index) {
        items.remove(index);
    }
    
    /**
     * Method for getting an item.
     * 
     * @param index The index
     * @return The item
     */
    public Item getItem(int index) {
        return ItemList.INSTANCE.getItem(getItemListIndex(index));
    }
    
    /**
     * Method for getting all items.
     * 
     * @return List of items
     */
    public ArrayList getItems() {
        return items;
    }
    
    /**
     * Method for getting the number of items.
     * 
     * @return Number of items
     */
    public int size() {
        return items.size();
    }
    
    /**
     * Method for testin if the itemset contains a given item.
     * 
     * @param item The item
     * @return True if item is contained in itemset
     */
    public boolean containsItem(Item item) {
        return items.contains(new Integer(ItemList.INSTANCE.indexOfItem(item)));
    }
    
    /**
     * Method for comapring two itemsets.
     */
    public boolean equals(Object o) {
        int count = 0;
        ItemSet set;
        
        if (o instanceof ItemSet) {
            set = (ItemSet) o;
            for (int i = 0; i < set.size(); i++) {
                if (containsItem(set.getItem(i))) {
                    count++;
                }
            }
            return count == set.size();
        }
        return false;
    }
    
    /**
     * Method for updating count on itemset. count++;
     */
    public void addOne() {
        count++;
    }
    
    /**
     * Method for getting count.
     * 
     * @return The count
     */
    public int getCount() {
        return count;
    }
    
    /**
     * Method for setting the support.
     * 
     * @param support The support
     */
    public void setSupport(double support) {
        this.support = support;
    }
    
    /**
     * Method for getting the support.
     * 
     * @return The support
     */
    public double getSupport() {
        return support;
    }
    
    /**
     * Method for getting a clone of this instance.
     */
    public ItemSet clone() {
        ItemSet clone = new ItemSet();
        for (int i = 0; i < items.size(); i++) {
            clone.addItem((Item)((Item)getItem(i)).clone());
        }
        return clone;
    }
    
    /**
     * Method for getting a printable representation of this itemset.
     */
    public String toString() {
        String string = "[";
        
        for (int i = 0; i < size(); i++) {            
            string += getItem(i).toString() + " ";
        }
        return string + "]";
    }
}
