package modelling.fiti.data;

import java.util.ArrayList;

/**
 * Class containing a list of all unique terms used. Term index can be 
 * calculated as the index of term in the list. 
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class TermList {

    private ArrayList terms;
    
    /**
     * Constructor creating a new term list.
     */
    public TermList() {
        terms = new ArrayList();
    }
    
    /**
     * Method for adding a term. Only new terms are added.
     * 
     * @param term The term
     */
    public void addTerm(String term) {
        if (!terms.contains(term.toLowerCase())) {
            terms.add(term.toLowerCase());
        }        
    }
    
    /**
     * Method for getting the number of unique terms.
     * 
     * @return The number of unique terms
     */
    public int nofTerms() {
        return terms.size();
    }
    
    /**
     * Method for getting the index of a term.
     * 
     * @param term The term
     * @return The index
     */
    public int indexOf(String term) {
        return terms.indexOf(term.toLowerCase()) + 1;
    }
    
    /**
     * Method for getting a term by its index.
     * 
     * @param index The index
     * @return The term
     */
    public String getTerm(int index) {
        return (String)terms.get(index - 1);
    }
    
    /**
     * Method for getting a printable representation of the term list.
     */
    public String toString() {
        String string =  "Terms: (" + terms.size() + ")\n";
        
        for (int i = 0; i < terms.size(); i++) {
            string += "'" + (String)terms.get(i) + "'\n";
        }
        return string;
    }
}
