package modelling.fiti.data;

import java.util.ArrayList;

/**
 * Class implementing a transaction. A transaction is a set of items 
 * occurring togehter at a specific time.
 * 
 * Mega transactions have timestamp = -1.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class Transaction {

    private int timestamp;
    private ArrayList items;

    
    /**
     * Constructor creating a new transaction.
     * 
     * @param timestamp The timestamp
     */
    public Transaction(int timestamp) {
        this.timestamp = timestamp;
        items = new ArrayList();
    }
    
    /**
     * Constructor creating a new transaction. Timestamp is set to -1.
     */
    public Transaction() {
        timestamp = -1;
        items = new ArrayList();
    }
    
    /**
     * Method for adding an item to the transaction if it is not already 
     * present.
     * 
     * @param item The item
     */
    public void addItem(Object item) {
        if (!items.contains(item)) {
            items.add(item);
        }
    }
    
    /**
     * Method for getting the items in the transaction.
     * 
     * @return List of items
     */
    public ArrayList getItems() {
        return items;
    }

    /**
     * Method for finding if the transaction contains a specific itemset, i.e. 
     * all items in the itemset.
     * 
     * @param itemset The itemset
     * @return True if itemset is contained in the transaction
     */
    public boolean contains(ItemSet itemset) { 
        int count = 0;
        
        for (Object o : items) {
            for (int i = 0; i < itemset.size(); i++) {
                InterItem item1 = (InterItem)o;
                InterItem item2 = (InterItem)itemset.getItem(i);
                
                if (item1.equals(item2)) {
                    count++;
                }
            }
        }
        return count == itemset.getItems().size();
    }
    
    /**
     * Method for getting the timestamp of the transaction.
     * 
     * @return The timestamp
     */
    public int getTimestamp() {
        return timestamp;
    }
    
    /**
     * Method for getting a printable representation of the transaction.
     */
    public String toString() {
        String string  = "";
        
        string += "(" + timestamp + ") ";
        for (Object o : items) {
            string += o.toString() + " ";
        }
        return string;
    }   
}
