package modelling.fiti.helpers;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import gui.MainFrame;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Class implementing a tool for monitoring the memory usage. This class
 * implements a window showing time, memory usage, and maximum memory usage.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class StatusHelper extends Thread {

    //Waiting time, between each poll
    private static final int WAIT_TIME = 1000;
    
    private boolean running;
    private Runtime runtime;
    private OutDialog out;
    
    /**
     * Constructor creating a new instance of this class.
     */
    public StatusHelper() {
        // Create out window
        out = new OutDialog();
        
        //Start thread
        runtime = Runtime.getRuntime();
        running = true;
        start();
    }
    
    /**
     * Method for signalling that the the monitoring is finished.
     */
    public void finish() {
        running = false;
    }
    
    /**
     * Run method, updating the values.
     */
    public void run() {
        int counter = 0;
        
        while(running) {
            out.update(counter, runtime.totalMemory() - runtime.freeMemory(), 
                    runtime.totalMemory());
            counter++;
            try {
                Thread.sleep(WAIT_TIME);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Internal class, implementing the dialog showing the monitoring.
     * 
     * @author Kjell-Inge Skogstad and Trond ivind Eriksen
     */
    private class OutDialog extends JDialog {
        
        private JTextField time;
        private JTextField maxTime;
        private JTextField mem;
        private JTextField maxMem;
        private JTextField max;
        
        private long tempMax;
        
        /**
         * Constructor creating a new dialog.
         */
        public OutDialog() {
            super(MainFrame.getInstance(), "Memory status", false);
            
            tempMax = 0;
            
            addComponents();
            
            setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            pack();
            setLocationRelativeTo(MainFrame.getInstance());
            setVisible(true);
        }
        
        /**
         * Method adding all components to this dialog.
         */
        private void addComponents() {
            JPanel panel = new JPanel(new GridLayout(3, 4));
            
            max = new JTextField();
            max.setEditable(false);
            time = new JTextField();
            time.setEditable(false);
            maxTime = new JTextField();
            maxTime.setEditable(false);
            mem = new JTextField();
            mem.setEditable(false);
            maxMem = new JTextField();
            maxMem.setEditable(false);
            
            panel.add(new JLabel("Time:"));
            panel.add(time);
            panel.add(new JLabel("Memory Used:"));
            panel.add(mem);
            panel.add(new JLabel("Max Time:"));
            panel.add(maxTime);
            panel.add(new JLabel("Max Used:"));
            panel.add(maxMem);
            panel.add(new JLabel("Max Available:"));
            panel.add(max);
            
            panel.setPreferredSize(new Dimension(350, 90));
            
            getContentPane().setLayout(new BorderLayout());
            getContentPane().add(panel, BorderLayout.CENTER);
            
        }
        
        /**
         * Method used to update the values shown in this dialog.
         * 
         * @param time The time
         * @param memory The memory usage
         * @param maxMemory The maximum memory available
         */
        public void update(int time, long memory, long maxMemory) {
            this.time.setText(time + "");
            mem.setText((memory / 1000) + "k");
            max.setText((maxMemory / 1000) + "k");
            
            if (memory > tempMax) {
                tempMax = memory;
                maxTime.setText(time + "");
                maxMem.setText((memory / 1000) + "k");
            }
        }
    }
}
