package modelling.fiti.report;

import java.awt.BorderLayout;
import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;

import external.TableSorter;

import gui.ReportView;

/**
 * Class implementing the report of this operation. Presents the result in a 
 * new window.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class FitiReport extends ReportView {

    private String report;
    private RuleModel model;
    private TableSorter sorter;

    private JTable table;
    
    /**
     * Constructor generating a new report window.
     * 
     * @param report
     * @param model
     */
    public FitiReport(String report, RuleModel model) {
        this.report = report;
        this.model = model;
        
        addComponents();

    }

    /**
     * Method used to add all components.
     */
    public void addComponents() {
        JScrollPane scroll = new JScrollPane(
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        //JTable table;
        TableColumn column;
        JTableHeader header;
        /* Adding support for sorting */
        sorter = new TableSorter(model);
        table = new JTable(sorter);

        header = table.getTableHeader();
        sorter.setTableHeader(header);        
        /* Setting column size */ 
        for (int j = 0; j < model.getColumnCount(); j++) {
            column = table.getColumnModel().getColumn(j);
                    
            if (j == 0) {
                column.setPreferredWidth(200); 
            } else {
                column.setPreferredWidth(50);
            }
        }
        
        scroll.setViewportView(table);

        setLayout(new BorderLayout());
        add(scroll, BorderLayout.CENTER);
    }
    
    /**
     * Method returning a printable copy of the report.
     */
    public String getPrintableCopy() {
        String report = this.report;
        sorter.setSortingStatus(3, 1);
        /* Adding sorted rules */
        report += "Rules: (" + sorter.getRowCount() + ")\n";
        for (int i = 0; i < sorter.getRowCount(); i++) {
            report += "Rule " + (i + 1) + ":   ";
            report += sorter.getValueAt(i, 0) + " ,  ";
            report += "support = " + sorter.getValueAt(i, 1) + " ,  ";
            report += "confidence = " + sorter.getValueAt(i, 2) + " ,  ";
            report += "similarity = " +sorter.getValueAt(i, 3) +"\n";
        }
        return report;
    }

    public JTable getTable(){
        return table;
    }
}
