package modelling.fiti.report;

import java.util.ArrayList;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import modelling.fiti.Rule;

/**
 * Class implementing a table model for showing association rules.
 * 
 * @author Kjell-Inge Skogstad and Trond ivind Eriksen
 */
public class RuleModel implements TableModel {

    private ArrayList rules;
    
    /**
     * Constructor creating a new model.
     * 
     * @param rules List of rules
     */
    public RuleModel(ArrayList rules) {
        this.rules = rules;
    }
    
    /**
     * Method for gettin number of rules.
     */
    public int getRowCount() {
        return rules.size();
    }

    /**
     * Method for gettin number of columns.
     */
    public int getColumnCount() {
        return 4;
    }

    /**
     * Method for getting name of given column.
     */
    public String getColumnName(int arg0) {
        String name = "";
        
        switch (arg0) {
            case 0:
                name = "Rule";
                break;
            case 1:
                name = "Support";
                break;
            case 2:
                name = "Confidence";
                break;
            case 3:
                name = "Semantic Similarity";
                break;
        }
        return name;
    }

    /**
     * Method for getting class type of given column.
     */
    public Class<?> getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    /**
     * Method for getting if the cell is editable.
     */
    public boolean isCellEditable(int arg0, int arg1) {
        return false;
    }

    /**
     * Method for getting the value at given cell.
     */
    public Object getValueAt(int row, int col) {
        switch (col) {
            case 0: 
                return ((Rule)rules.get(row)).getRule();
            case 1: 
                return ((Rule)rules.get(row)).getSupport();
            case 2:
                return ((Rule)rules.get(row)).getConfidence();
            case 3:    
                return ((Rule)rules.get(row)).getSimilarity();
        }
        return null;
    }

    /**
     * Method for setting the value at given cell. Not implemented.
     */
    public void setValueAt(Object arg0, int arg1, int arg2) {
    }

    /**
     * Method for adding a table model listener. Not implemented.
     */
    public void addTableModelListener(TableModelListener arg0) {
    }

    /**
     * Method for removing a table model listener. Not implemented.
     */
    public void removeTableModelListener(TableModelListener arg0) {
    }
}
