package newsloader;

import datapreparation.stopword.Stopwords;
import datapreparation.stopword.EnglishStopwords;

import java.io.*;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Arrays;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CodingErrorAction;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;

/**
 * A class that reads the contents of a file, and extracts news items.
 *
 * @author Ole Kristian Fivelstad
 */

public class Extractor extends Thread {
    private File directory;
    private File outputFile;
    private File[] files;

    private ArrayList globalHashKeys;
    private ArrayList hashValues;
    private ArrayList globalNews;

    private ArrayList<String> newsItems;
    private int noOfFiles;
    public int items=0;
    private BufferedWriter out;

    private String dataSetName;
    private String charSet="";

    private NewsLoader newsloader;

    private boolean financialTimes=false;
    private boolean bbcNews=false;


    /**
     * Constructor for the extractor class.
     *
     * @param url Location of the directory
     * @param output File to store the result in
     * @param dataSetName Name of the dataset
     */
    public Extractor(String url, String output, String dataSetName) {

        newsloader=NewsLoader.getInstance();
        this.dataSetName=dataSetName;
        globalHashKeys = new ArrayList();
        globalNews = new ArrayList();
        outputFile=new File(output);
        directory=new File(url);

        if(directory.isDirectory()){
            files=directory.listFiles();
            Arrays.sort(files);
            noOfFiles=files.length;
        }

    }

    public void run(){
        //using first file in the dataset to detect which characterset is used
        setCharset(files[1]);


        for(int i = 0; i<noOfFiles; i++){
            File temp = files[i];            
            readFile(temp);
            int progress = (int)((double)(i + 1)/(double)files.length * 100);
            newsloader.updateProgress(progress);

        }
        writeXML();
    }

    public void setCharset(File file){
        try {
            String prefix = "file:///";
            prefix =prefix +file.getAbsolutePath();
            URL fileUrl = new URL(prefix);
            URLConnection c = fileUrl.openConnection();

            int bytesRead = 0;
            int contentLen = c.getContentLength();
            byte[] b = new byte[contentLen];
            InputStream is = c.getInputStream();

            while (bytesRead < contentLen) {
                int len = is.read(b, bytesRead, contentLen - bytesRead);
                if (len == -1) {
                    break;
                }
                bytesRead += len;
            }
            String contents = new String(b, 0, bytesRead);
            String temp = contents.toLowerCase();

            if(temp.indexOf("financial times")!=-1||temp.indexOf("ft.com")!=-1){
                System.out.println("Financial Times detected...");
                financialTimes=true;
            } else if (temp.indexOf("bbc news")!=-1){
                System.out.println("BBC Finance detected...");
                bbcNews=true;
            }

            int charsetIndex = temp.indexOf("charset=");
            if(charsetIndex!=-1){
                int charsetEnd=temp.indexOf("\"", charsetIndex);
                charSet=temp.substring(charsetIndex+8, charsetEnd).trim();
                charSet=charSet.toUpperCase();
            } else{
                charSet="ISO-8859-1";
            }
            is.close();
        } catch(Exception e){

        }
    }

    public String prepareFinancialTimes(String contents){
        String temp = contents.toLowerCase();
        int start=temp.indexOf("principal content");
        int end;
        if(start!=-1){
            int tempEnd=temp.indexOf("home europe", start);
            end = temp.indexOf("</a", tempEnd);
            end = temp.indexOf(">", end)+1;
            if(end!=-1&&end>start){
                contents=contents.substring(start, end);
            } else {
                contents = contents.substring(start, contents.length());
            }
        }
        temp=null;
        return contents;
    }

    public String prepareBbcNews(String contents){
        contents=contents.replaceAll("More Video", "");
        String temp = contents.toLowerCase();
        int start=temp.indexOf("news front page");
        int end;
        if(start!=-1){
            end=temp.indexOf("from other bbc sites", start);
            if(end!=-1){
                contents=contents.substring(start, end);
                String temp2=contents.toLowerCase();
                int marketData = temp2.indexOf("marketdatah");
                if(marketData!=-1){
                    int marketDataEnd = temp2.indexOf("features, views, analysis", marketData);
                    if(marketDataEnd!=-1){
                        String temp3=contents.substring(0, marketData);
                        String temp4=contents.substring(marketDataEnd, contents.length());
                        contents=temp3+" "+temp4;
                        temp3=null;
                        temp4=null;
                    }
                } 
                temp2=null;
            }
        }
        temp=null;
        return contents;
    }


    /**
     * Method for reading a file
     */
    public void readFile(File file){
        newsItems=null;



        try {
            String prefix = "file:///";
            prefix =prefix +file.getAbsolutePath();
            URL fileUrl = new URL(prefix);
            URLConnection c = fileUrl.openConnection();

            int bytesRead = 0;
            int contentLen = c.getContentLength();
            byte[] b = new byte[contentLen];
            char[] ch = new char[contentLen];
            InputStream is = c.getInputStream();
            InputStreamReader isr = new InputStreamReader(is, charSet);
            BufferedReader br = new BufferedReader(isr);
            int numRead;
            while ((numRead = br.read(ch, 0, contentLen)) != -1);
            br.close();
            String contents = new String(ch);
            is.close();
            extractNewsItems(contents);

        } catch(IOException e) {
            System.out.println("Something went wrong when trying to read file... \n"+e);
            e.printStackTrace();
        }

    }

    /**
     * Method for performing news item extraction
     *
     * @param content The content of a file
     */
    private void extractNewsItems(String content){
        hashValues = new ArrayList();

        String webpage = content;
        newsItems = new ArrayList();

        if(financialTimes){
            webpage=prepareFinancialTimes(webpage);
        } else if(bbcNews){
            webpage=prepareBbcNews(webpage);
        } else{
            // leave webpage as is...
        }
        webpage=cleanWebpage(webpage);
        webpage = urlTextUrlItem(webpage);
        webpage = anchorTextItem(webpage);
        webpage = textBasedItem(webpage);
        globalHashKeys = hashValues;
        if(newsItems.isEmpty()){
            newsItems.add("");
        }
        globalNews.add(newsItems);
        newsItems=null;
    }

    /**
     * Method for finding url-text-url items
     *
     * @param content The content of the webpage
     * @return content after url-text-url items have been found
     */
    private String urlTextUrlItem(String content) {
        boolean utuItem = true;
        int utuIndex=0;

        while(utuItem){
            String newsItem="";
            int startUrl = content.indexOf("<a href=",utuIndex);
            int endUrl;
            if(startUrl==-1){
                utuItem=false;
                break;
            }
            endUrl=content.indexOf(">",startUrl);
            int endTag = content.indexOf("</a>",endUrl);
            utuIndex=endTag+4;
            String url = content.substring(startUrl,endUrl);
            if(url.indexOf("class")!=-1){
                url=url.substring(0,url.indexOf("class"));
            }
            String headline = content.substring(endUrl+1,endTag);
            StringTokenizer st = new StringTokenizer(headline);
            if(endTag==-1){
                utuItem=false;
                break;
            }
            int nextUrlTag = content.indexOf("<a href=", endTag);
            int endNextUrlTag = content.indexOf(">",nextUrlTag);
            if(nextUrlTag==-1||endNextUrlTag==-1){
                utuItem=false;
                break;
            }
            String url2 = content.substring(nextUrlTag, endNextUrlTag);
            if(url2.indexOf("class")!=-1){
                url2=url2.substring(0, url2.indexOf("class"));
            }
            String urlTextUrl = content.substring(startUrl,endNextUrlTag+1);
            String newscontent="";
            //checking to see whether the urls are equal
            if(url.equalsIgnoreCase(url2)){
                if(st.countTokens()>=3){
                    newsItem+=headline+".";
                }
                newscontent=content.substring(endTag+4,nextUrlTag);
                newscontent=removeTags(newscontent);
                st=new StringTokenizer(newscontent);
                if(st.countTokens()>5){
                    newsItem+=" "+newscontent;
                    hashValues.add(newsItem.hashCode());
                    if(!globalHashKeys.contains(newsItem.hashCode())){
                        utuIndex=startUrl;
                        content=content.replace(urlTextUrl,"");
                        newsItem=removeTags(newsItem);
                        newsItems.add(newsItem);
                    }
                    else{
                        //remove the urlTextUrl-item so that it does not get
                        //counted as anchor- or text-item
                        content=content.replace(urlTextUrl,"");
                    }
                }
            }
            else{

            }
        }

        return content;
    }

    /**
     * Method for getting the number of files
     * @return number of files
     */
    public int getNoOfFiles(){
        return noOfFiles;
    }

    /**
     * Method for finding text-based items
     *
     * @param content The content of the webpage
     * @return content after text-based items have been found
     */
    public String textBasedItem(String content){
        boolean text = true;
        int textIndex=0;
        while(text){
            int start = content.indexOf(">",textIndex);
            if(start!=-1){
                int end = content.indexOf("<",start);
                if(end!=-1){
                    String news = content.substring(start+1,end);
                    String newsItem =removeTags(news);
                    StringTokenizer st = new StringTokenizer(newsItem);
                    //if string contains more than 10 words it is most likely a news item
                    if(st.countTokens()>10&&!hashValues.contains(newsItem.hashCode())
                            &&newsItem.indexOf("{")==-1){
                        hashValues.add(newsItem.hashCode());
                        if(!globalHashKeys.contains(newsItem.hashCode())){
                            newsItems.add(newsItem);
                            content=content.replaceAll(news,"");
                        }
                    }
                    textIndex=end+1;
                }
                else{
                    textIndex=start+1;
                }
                text=true;
            }
            else{
                text = false;
            }
        }
        return content;
    }

    /**
     * Method for finding anchor-textx items
     *
     * @param content The content of a webpage
     * @return content after anchor-text items have been found
     */
    public String anchorTextItem(String content){
        boolean anchor=true;
        int urlIndex = 0;

        while(anchor){
            int startUrl = content.indexOf("<A",urlIndex);
            int startUrlSmall = content.indexOf("<a",urlIndex);
            int endStartUrl;
            int endUrl;
            if(startUrl==-1&&startUrlSmall==-1){
                anchor=false;
                break;
            }
            else if(startUrlSmall!=-1&&startUrl==-1){
                startUrl=startUrlSmall;
            }
            else if(startUrlSmall!=-1&&startUrl!=-1&&startUrlSmall<startUrl){
                startUrl=startUrlSmall;
            }
            endStartUrl=content.indexOf(">",startUrl);
            int endUrlSmall = content.indexOf("</a>",startUrl);
            endUrl=content.indexOf("</A>",startUrl);
            if(endUrlSmall!=-1&&endUrl!=-1&&endUrlSmall<endUrl){
                endUrl=endUrlSmall;
            }
            else if(endUrl==-1&&endUrlSmall!=-1){
                endUrl=endUrlSmall;
            }
            String newsitem=content.substring(endStartUrl+1,endUrl);
            newsitem=removeTags(newsitem);
            StringTokenizer st = new StringTokenizer(newsitem);

            //if string contains more than 4 items it is most likely a news item
            if(st.countTokens()>4&&(newsitem.indexOf("<img")==-1)
                    &&newsitem.indexOf("<IMG")==-1&&!hashValues.contains(newsitem.hashCode())
                    &&newsitem.indexOf("{")==-1){
                hashValues.add(newsitem.hashCode());
                if(!globalHashKeys.contains(newsitem.hashCode())){
                    if(!newsitem.endsWith(".")&&!newsitem.endsWith("!")&&!newsitem.endsWith("?")){
                        newsitem=newsitem+".";
                    }
                    newsItems.add(newsitem);
                    content=content.replaceAll(content.substring(endStartUrl+1,endUrl),"");
                }
            }
            urlIndex=endUrl+4;
        }


        return content;
    }

    /**
     * Method for removing scripts from a webpage
     * @param content The content of a webpage
     * @return content after scripts have been removed
     */
    public String cleanWebpage(String content){
        boolean script=true;

        while(script){
            int scriptStart=content.indexOf("<SCRIPT");
            int scriptStartSmall=content.indexOf("<script");

            if(scriptStart==-1&&scriptStartSmall==-1){
                script=false;
            }
            else if(scriptStart==-1&&scriptStartSmall!=-1){
                scriptStart=scriptStartSmall;
            }
            else if(scriptStart!=-1&&scriptStartSmall!=-1&&scriptStartSmall<scriptStart){
                scriptStart=scriptStartSmall;
            }

            if(script){
                int scriptEnd=content.indexOf("</SCRIPT>",scriptStart);
                int scriptEndSmall=content.indexOf("</script>",scriptStart);
                if(scriptEnd==-1&&scriptEndSmall!=-1){
                    scriptEnd=scriptEndSmall;
                }
                else if(scriptEnd!=-1&&scriptEndSmall!=-1&&scriptEndSmall<scriptEnd){
                    scriptEnd=scriptEndSmall;
                }
                if(scriptEnd!=-1){
                    content=(content.substring(0,scriptStart)).concat(content.substring(scriptEnd+9,content.length()));
                }

            }
        }

        return content;
    }

    /**
     * Method for removing HTML tags from a string
     * @param string The string to remove tags from
     * @return string without HTML tags
     */
    public String removeTags(String string){
        String cleaned = string;
        boolean cleaning = true;
        while(cleaning){
            int startTag = cleaned.indexOf("<");
            if(startTag==-1){
                cleaning=false;
            }
            else{
                int endTag = cleaned.indexOf(">");
                String dateClassStart = cleaned.substring(startTag,endTag+1);
                if(dateClassStart.equals("<span class=dateGrey>")){
                    int endClass = cleaned.indexOf("</span>", endTag+1);
                    if(endClass!=-1){
                        String tag2=cleaned.substring(startTag,endClass+7);
                        cleaned=cleaned.replace(tag2," ");
                    }
                }
                else{
                    String tag = cleaned.substring(startTag,endTag+1);
                    if(tag.toLowerCase().contains("<br")||tag.toLowerCase().contains("<p")){
                        cleaned=cleaned.replace(tag, ". ");
                    } else {
                        cleaned=cleaned.replace(tag,"");
                    }
                }
            }
        }

        cleaned=cleaned.replaceAll("\n","");
        cleaned=cleaned.replaceAll("&amp;","");
        cleaned=cleaned.replaceAll("&quot;","\"");
        //cleaned=cleaned.replaceAll("\\|","");

        cleaned=cleaned.replaceAll("&nbsp;","");
        cleaned=cleaned.replaceAll("&aring;","");
        cleaned=cleaned.replaceAll("&","");
        cleaned=cleaned.replaceAll("&lt;", " ");
        cleaned=cleaned.replaceAll("&gt;", " ");
        cleaned=cleaned.replaceAll("&copy;", " ");
        cleaned=cleaned.replaceAll("&#8364;", " ");
        cleaned=cleaned.replaceAll("&#8211;", " ");
        cleaned=cleaned.replaceAll("&#8217;", " ");
        //cleaned=cleaned.replaceAll("’","'");
        //cleaned=cleaned.replaceAll("–","-");
        cleaned=cleaned.replaceAll("&#150", " ");
        cleaned=cleaned.replaceAll("Q&A:", "");



        return cleaned.trim();
    }

    /**
     * Method for writing the result to a XML-file
     */
    public void writeXML(){

        try {
            FileOutputStream fos = new FileOutputStream(outputFile);
            OutputStreamWriter osw = new OutputStreamWriter(fos, charSet);
            //out=new BufferedWriter(new FileWriter(outputFile));
            out=new BufferedWriter(osw);
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            out.write("<?xml version=\"1.0\" encoding=\""+charSet+"\"?>"); //
            out.newLine();
            out.write("<tokenized>");
            out.newLine();
            out.newLine();
            out.write("<dataSet name=\""+dataSetName+"\" time=\"days\" language=\"English\">");
            out.newLine();
            /*
            if(clustering){
                clusterDir = new File("ClusterFiles"+File.separator);
                clusterDir.mkdir();
            }*/
            for(int i = 0;i<globalNews.size(); i++){
                ArrayList news = (ArrayList)globalNews.get(i);
                for(int j = 0; j<news.size();j++){
                    out.write("<text tid=\""+(i+1)+"\">");
                    out.newLine();
                    String newsItem = (String)news.get(j);

                    out.write(newsItem);
                    //out.write((String)news.get(j));
                    out.newLine();
                    out.write("</text>");
                    out.newLine();
                }
                /*
                if(clustering){
                    writeClusterFiles(news);
                }
                */
                news=null;
            }
            out.write("</dataSet>");
            out.newLine();
            out.write("</tokenized>");
            out.close();
            newsloader.updateProgress(101);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}


