package newsloader;

import org.jdom.input.SAXBuilder;
import org.jdom.Document;
import org.jdom.Element;
import newsloader.Extractor;
import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.io.File;
import java.nio.charset.Charset;

/**
 * The class responsible for showing the GUI of the news item extractor. The user
 * can specify a config file which is read by this class.
 *
 * @author Ole Kristian Fivelstad
 *
 */

public class NewsLoader extends JFrame {

    private double noOfFiles=0;
    private double counter=1;


    private Extractor ex;

    private JTextField jtf;
    private JTextField jtf2;

    private JButton button;
    private JButton button2;
    private JButton button3;

    private JTextField prog;

    private JProgressBar jprog;

    //private String configUrl="";
    //private String url = "";
    private String directory="";
    private String output = "";
    private String datasetName="";




    private boolean outputSet=false;
    private boolean directorySelected=false;





    private static NewsLoader INSTANCE;


    /**
     * Constructor for the newsloader class.
     *
     */
    public NewsLoader(){
        super("NewsLoader");
        INSTANCE=this;
        addComponents();
        addListeners();
        setSize(350,200);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationRelativeTo(null);
        setVisible(true);

    }

    public static NewsLoader getInstance(){
        return INSTANCE;
    }

    /**
     * Method for adding components to the GUI
     */
    public void addComponents(){
        JPanel panel = new JPanel(new GridLayout(6,1));

        prog = new JTextField();
        prog.setText("Not started");
        prog.setEditable(false);

        jprog = new JProgressBar(0, 100);
        jprog.setStringPainted(true);
        jprog.setString("0%");
        jprog.setValue(0);

        button = new JButton("Start process");


        jtf = new JTextField();
        jtf.setText("No folder has been selected");
        jtf.setEditable(false);

        jtf2 = new JTextField();
        jtf2.setText("No output file has been selected");
        jtf2.setEditable(false);

        button2 = new JButton("Select folder");

        button3 = new JButton("Select output file");

        panel.add(jprog);
        panel.add(button);
        panel.add(jtf);
        panel.add(button2);
        panel.add(jtf2);
        panel.add(button3);
        getContentPane().add(panel);
    }

    /**
     * Method for adding listeners to the buttons
     */
    public void addListeners(){
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                if(!outputSet||!directorySelected){
                    JOptionPane.showMessageDialog (null,
                            "You must select both directory and output file first", "Warning",
                    JOptionPane.WARNING_MESSAGE);
                }
                else{
                    button.setEnabled(false);
                    ex=new Extractor(directory, output, datasetName);
                    ex.start();
                    /*
                    File config = new File(configUrl);
                    String url2=config.getAbsolutePath();
                    try {
                        readConfig(url2);
                        System.out.println(url);
                    } catch (Exception e1) {
                        e1.printStackTrace();
                    }
                    ex = new Extractor(url, output, dataSetName);
                    noOfFiles=ex.getNoOfFiles();
                    if(!running){
                        startAnimation();
                    }*/
                }
            }

        });

        /*
        button2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                JFileChooser jfc = new JFileChooser();
                jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                jfc.showOpenDialog(null);
                File selFile = jfc.getSelectedFile();
                if(selFile!=null){
                    configUrl=selFile.getPath();
                    jtf.setText(configUrl);
                    readConfig=true;
                }
            }
        });*/
        button2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                JFileChooser jfc = new JFileChooser();
                jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                jfc.showOpenDialog(null);
                File selFile = jfc.getSelectedFile();
                if(selFile!=null){
                    directory=selFile.getAbsolutePath();
                    datasetName=selFile.getName();
                    jtf.setText(directory);
                    directorySelected=true;
                }
            }
        });

        button3.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                JFileChooser jfc = new JFileChooser();
                jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                jfc.showOpenDialog(null);
                File selFile = jfc.getSelectedFile();
                if(selFile!=null){
                    output=selFile.getAbsolutePath();
                    jtf2.setText(output);
                    outputSet=true;
                }
            }
        });
    }

    /**
     * Method used to update the progress bar. The progress is given by a
     * number between 0 and 100.
     *
     * @param newProgress The new progress
     */
    public void updateProgress(int newProgress) {
        if (newProgress <= 100 && newProgress >= 0) {
            jprog.setString(newProgress + "%");
            jprog.setValue(newProgress);
            jprog.revalidate();
            jprog.repaint();
        } else if(newProgress>100){
            jprog.setString("Finished...");
            jprog.setValue(100);
            jprog.revalidate();
            jprog.repaint();
        }
    }

    /**
     * Method for reading the config file
     * @param url The location of the file
     */
    /*
    public void readConfig(String url){
        SAXBuilder sb = new SAXBuilder();
        Document doc = null;
        try {
            doc = sb.build(url);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("The directory path probably contains ,  or ");
            System.out.println("The news items are still extracted...");
        }
        Element root = doc.getRootElement();
        Element collection = root.getChild("collection");
        Element directory = collection.getChild("directory");
        this.url=directory.getValue();
        Element name = collection.getChild("dataSetName");
        dataSetName = (name.getValue()).trim();
        Element outputFile = collection.getChild("output");
        output = outputFile.getValue();
        Element properties = root.getChild("tokenizer");
        String urlTextUrlSel=properties.getChild("urltexturl").getValue();
        String anchorSel=properties.getChild("anchoritem").getValue();
        String textSel=properties.getChild("textitem").getValue();
        urlTextUrlItem=Boolean.valueOf(urlTextUrlSel);
        anchorItem=Boolean.valueOf(anchorSel);
        textItem=Boolean.valueOf(textSel);
        readConfig=true;
    }*/


    /**
     * Thread which notifies the extractor to read a file. Responsible
     * for updating progress.
     */
    /*
    public void run() {
        double percentage;
        int progress=0;
        while(running){
            ex.readFile();
            percentage =100;//(counter/(noOfFiles))*100;
            progress=(int)percentage;
            if(progress>prevProgress){
                prog.setText(progress+"% done...");
                prevProgress=progress;
            }
            counter++;
            if(progress==100){
                running=false;
                System.out.println("Writing xml...");
                ex.writeXML();
                System.out.println("Xml has been written");
            }
        }
    }
    */
}
